@@ -1,11 +1,11 @@
 from ..utils import now
-from .forms import ClientAuthForm
+from .forms import ClientAuthForm, PublicPasswordGrantForm
 from .models import AccessToken
 
 
 class BaseBackend(object):
     """
-    Base backend used to authenticate clients as defined in :draft:`1` against
+    Base backend used to authenticate clients as defined in :rfc:`1` against
     our database.
     """
     def authenticate(self, request=None):
@@ -19,7 +19,7 @@ class BaseBackend(object):
 class BasicClientBackend(object):
     """
     Backend that tries to authenticate a client through HTTP authorization
-    headers as defined in :draft:`2.3.1`.
+    headers as defined in :rfc:`2.3.1`.
     """
     def authenticate(self, request=None):
         auth = request.META.get('HTTP_AUTHORIZATION')
@@ -47,7 +47,7 @@ class BasicClientBackend(object):
 class RequestParamsClientBackend(object):
     """
     Backend that tries to authenticate a client through request parameters
-    which might be in the request body or URI as defined in :draft:`2.3.1`.
+    which might be in the request body or URI as defined in :rfc:`2.3.1`.
     """
     def authenticate(self, request=None):
         if request is None:
@@ -61,6 +61,27 @@ class RequestParamsClientBackend(object):
         return None
 
 
+class PublicPasswordBackend(object):
+    """
+    Backend that tries to authenticate a client using username, password
+    and client ID. This is only available in specific circumstances:
+
+     - grant_type is "password"
+     - client.client_type is 'public'
+    """
+
+    def authenticate(self, request=None):
+        if request is None:
+            return None
+
+        form = PublicPasswordGrantForm(request.REQUEST)
+
+        if form.is_valid():
+            return form.cleaned_data.get('client')
+
+        return None
+
+
 class AccessTokenBackend(object):
     """
     Authenticate a user via access token and client object.
