@@ -48,7 +48,7 @@ class ClientAuthForm(forms.Form):
 class ScopeChoiceField(forms.ChoiceField):
     """
     Custom form field that seperates values on space as defined in
-    :draft:`3.3`.
+    :rfc:`3.3`.
     """
     widget = forms.SelectMultiple
 
@@ -56,8 +56,13 @@ class ScopeChoiceField(forms.ChoiceField):
         if not value:
             return []
 
+        # New in Django 1.6: value may come in as a string.
+        # Instead of raising an `OAuthValidationError`, try to parse and
+        # ultimately return an empty list if nothing remains -- this will
+        # eventually raise an `OAuthValidationError` in `validate` where
+        # it should be anyways.
         if not isinstance(value, (list, tuple)):
-            raise OAuthValidationError({'error': 'invalid_request'})
+            value = value.split(' ')
 
         # Split values into list
         return u' '.join([smart_unicode(val) for val in value]).split(u' ')
@@ -103,7 +108,7 @@ class AuthorizationRequestForm(ScopeMixin, OAuthForm):
     This form is used to validate the request data that the authorization
     endpoint receives from clients.
 
-    Included data is specified in :draft:`4.1.1`.
+    Included data is specified in :rfc:`4.1.1`.
     """
     # Setting all required fields to false to explicitly check by hand
     # and use custom error messages that can be reused in the OAuth2
@@ -132,7 +137,7 @@ class AuthorizationRequestForm(ScopeMixin, OAuthForm):
 
     def clean_response_type(self):
         """
-        :draft:`3.1.1` Lists of values are space delimited.
+        :rfc:`3.1.1` Lists of values are space delimited.
         """
         response_type = self.cleaned_data.get('response_type')
 
@@ -153,7 +158,7 @@ class AuthorizationRequestForm(ScopeMixin, OAuthForm):
 
     def clean_redirect_uri(self):
         """
-        :draft:`3.1.2` The redirect value has to match what was saved on the
+        :rfc:`3.1.2` The redirect value has to match what was saved on the
             authorization server.
         """
         redirect_uri = self.cleaned_data.get('redirect_uri')
@@ -301,3 +306,30 @@ class PasswordGrantForm(ScopeMixin, OAuthForm):
 
         data['user'] = user
         return data
+
+
+class PublicPasswordGrantForm(PasswordGrantForm):
+    client_id = forms.CharField(required=True)
+    grant_type = forms.CharField(required=True)
+
+    def clean_grant_type(self):
+        grant_type = self.cleaned_data.get('grant_type')
+
+        if grant_type != 'password':
+            raise OAuthValidationError({'error': 'invalid_grant'})
+
+        return grant_type
+
+    def clean(self):
+        data = super(PublicPasswordGrantForm, self).clean()
+
+        try:
+            client = Client.objects.get(client_id=data.get('client_id'))
+        except Client.DoesNotExist:
+            raise OAuthValidationError({'error': 'invalid_client'})
+
+        if client.client_type != 1: # public
+            raise OAuthValidationError({'error': 'invalid_client'})
+
+        data['client'] = client
+        return data
