@@ -4,6 +4,8 @@ from django.http import HttpResponse
 from django.http import HttpResponseRedirect, QueryDict
 from django.utils.translation import ugettext as _
 from django.views.generic.base import TemplateView
+from django.core.exceptions import ObjectDoesNotExist
+from oauth2.models import Client
 from . import constants, scope
 
 
@@ -22,8 +24,8 @@ class OAuthError(Exception):
 
         raise OAuthError({'error': 'invalid_request'})
 
-    The different types of errors are outlined in :draft:`4.2.2.1` and
-    :draft:`5.2`.
+    The different types of errors are outlined in :rfc:`4.2.2.1` and
+    :rfc:`5.2`.
 
     """
 
@@ -32,7 +34,7 @@ class OAuthView(TemplateView):
     """
     Base class for any view dealing with the OAuth flow. This class overrides
     the dispatch method of :attr:`TemplateView` to add no-caching headers to
-    every response as outlined in :draft:`5.1`.
+    every response as outlined in :rfc:`5.1`.
     """
 
     def dispatch(self, request, *args, **kwargs):
@@ -87,7 +89,7 @@ class Mixin(object):
 
 class Capture(OAuthView, Mixin):
     """
-    As stated in section :draft:`3.1.2.5` this view captures all the request
+    As stated in section :rfc:`3.1.2.5` this view captures all the request
     parameters and redirects to another URL to avoid any leakage of request
     parameters to potentially harmful JavaScripts.
 
@@ -103,8 +105,8 @@ class Capture(OAuthView, Mixin):
 
     def get_redirect_url(self, request):
         """
-        Return a redirect to a URL where the resource owner (see :draft:`1`)
-        authorizes the client (also :draft:`1`).
+        Return a redirect to a URL where the resource owner (see :rfc:`1`)
+        authorizes the client (also :rfc:`1`).
 
         :return: :class:`django.http.HttpResponseRedirect`
 
@@ -131,7 +133,7 @@ class Capture(OAuthView, Mixin):
 
 class Authorize(OAuthView, Mixin):
     """
-    View to handle the client authorization as outlined in :draft:`4`.
+    View to handle the client authorization as outlined in :rfc:`4`.
     Implementation must override a set of methods:
 
     * :attr:`get_redirect_url`
@@ -144,11 +146,11 @@ class Authorize(OAuthView, Mixin):
     display the authorization form.
 
     On successful authorization, it redirects the user back to the defined
-    client callback as defined in :draft:`4.1.2`.
+    client callback as defined in :rfc:`4.1.2`.
 
     On authorization fail :attr:`Authorize` displays an error message to the
     user with a modified redirect URL to the callback including the error
-    and possibly description of the error as defined in :draft:`4.1.2.1`.
+    and possibly description of the error as defined in :rfc:`4.1.2.1`.
     """
     template_name = 'provider/authorize.html'
 
@@ -188,7 +190,7 @@ class Authorize(OAuthView, Mixin):
         """
         Save the authorization that the user granted to the client, involving
         the creation of a time limited authorization code as outlined in
-        :draft:`4.1.2`.
+        :rfc:`4.1.2`.
 
         Should return ``None`` in case authorization is not granted.
         Should return a string representing the authorization code grant.
@@ -226,7 +228,7 @@ class Authorize(OAuthView, Mixin):
 
         :param request: :attr:`django.http.HttpRequest`
         :param error: ``dict``
-            The different types of errors are outlined in :draft:`4.2.2.1`
+            The different types of errors are outlined in :rfc:`4.2.2.1`
         """
         ctx = {}
         ctx.update(error)
@@ -259,7 +261,6 @@ class Authorize(OAuthView, Mixin):
         authorization_form = self.get_authorization_form(request, client,
             post_data, data)
 
-
         if not authorization_form.is_bound or not authorization_form.is_valid():
             return self.render_to_response({
                 'client': client,
@@ -269,9 +270,11 @@ class Authorize(OAuthView, Mixin):
         code = self.save_authorization(request, client,
             authorization_form, data)
 
+        # be sure to serialize any objects that aren't natively json
+        # serializable because these values are stored as session data
         self.cache_data(request, data)
         self.cache_data(request, code, "code")
-        self.cache_data(request, client, "client")
+        self.cache_data(request, client.serialize(), "client")
 
         return HttpResponseRedirect(self.get_redirect_url(request))
 
@@ -288,12 +291,33 @@ class Redirect(OAuthView, Mixin):
     This can be either parameters indicating success or parameters indicating
     an error.
     """
+
+    def error_response(self, error, mimetype='application/json', status=400,
+            **kwargs):
+        """
+        Return an error response to the client with default status code of
+        *400* stating the error as outlined in :rfc:`5.2`.
+        """
+        return HttpResponse(json.dumps(error), mimetype=mimetype,
+                status=status, **kwargs)
+
     def get(self, request):
         data = self.get_data(request)
         code = self.get_data(request, "code")
         error = self.get_data(request, "error")
         client = self.get_data(request, "client")
 
+        # client must be properly deserialized to become a valid instance
+        client = Client.deserialize(client)
+
+        # this is an edge case that is caused by making a request with no data
+        # it should only happen if this view is called manually, out of the
+        # normal capture-authorize-redirect flow.
+        if data is None or client is None:
+            return self.error_response({
+                'error': 'invalid_data',
+                'error_description': _('Data has not been captured')})
+
         redirect_uri = data.get('redirect_uri', None) or client.redirect_uri
 
         parsed = urlparse.urlparse(redirect_uri)
@@ -338,11 +362,11 @@ class AccessToken(OAuthView, Mixin):
     The default implementation supports the grant types defined in
     :attr:`grant_types`.
 
-    According to :draft:`4.4.2` this endpoint too must support secure
+    According to :rfc:`4.4.2` this endpoint too must support secure
     communication. For strict enforcement of secure communication at
     application level set :attr:`settings.OAUTH_ENFORCE_SECURE` to ``True``.
 
-    According to :draft:`3.2` we can only accept POST requests.
+    According to :rfc:`3.2` we can only accept POST requests.
 
     Returns with a status code of *400* in case of errors. *200* in case of
     success.
@@ -437,7 +461,7 @@ class AccessToken(OAuthView, Mixin):
             **kwargs):
         """
         Return an error response to the client with default status code of
-        *400* stating the error as outlined in :draft:`5.2`.
+        *400* stating the error as outlined in :rfc:`5.2`.
         """
         return HttpResponse(json.dumps(error), mimetype=mimetype,
                 status=status, **kwargs)
@@ -445,21 +469,32 @@ class AccessToken(OAuthView, Mixin):
     def access_token_response(self, access_token):
         """
         Returns a successful response after creating the access token
-        as defined in :draft:`5.1`.
+        as defined in :rfc:`5.1`.
         """
+
+        response_data = {
+            'access_token': access_token.token,
+            'token_type': constants.TOKEN_TYPE,
+            'expires_in': access_token.get_expire_delta(),
+            'scope': ' '.join(scope.names(access_token.scope)),
+        }
+
+        # Not all access_tokens are given a refresh_token
+        # (for example, public clients doing password auth)
+        try:
+            rt = access_token.refresh_token
+            response_data['refresh_token'] = rt.token
+        except ObjectDoesNotExist:
+            pass
+
         return HttpResponse(
-            json.dumps({
-                'access_token': access_token.token,
-                'expires_in': access_token.get_expire_delta(),
-                'refresh_token': access_token.refresh_token.token,
-                'scope': ' '.join(scope.names(access_token.scope)),
-            }), mimetype='application/json'
+            json.dumps(response_data), mimetype='application/json'
         )
 
     def authorization_code(self, request, data, client):
         """
         Handle ``grant_type=authorization_code`` requests as defined in
-        :draft:`4.1.3`.
+        :rfc:`4.1.3`.
         """
         grant = self.get_authorization_code_grant(request, request.POST,
                 client)
@@ -476,10 +511,11 @@ class AccessToken(OAuthView, Mixin):
 
     def refresh_token(self, request, data, client):
         """
-        Handle ``grant_type=refresh_token`` requests as defined in :draft:`6`.
+        Handle ``grant_type=refresh_token`` requests as defined in :rfc:`6`.
         """
         rt = self.get_refresh_token_grant(request, data, client)
 
+        # this must be called first in case we need to purge expired tokens
         self.invalidate_refresh_token(rt)
         self.invalidate_access_token(rt.access_token)
 
@@ -491,7 +527,7 @@ class AccessToken(OAuthView, Mixin):
 
     def password(self, request, data, client):
         """
-        Handle ``grant_type=password`` requests as defined in :draft:`4.3`.
+        Handle ``grant_type=password`` requests as defined in :rfc:`4.3`.
         """
 
         data = self.get_password_grant(request, data, client)
@@ -502,7 +538,9 @@ class AccessToken(OAuthView, Mixin):
             at = self.get_access_token(request, user, scope, client)
         else:
             at = self.create_access_token(request, user, scope, client)
-            rt = self.create_refresh_token(request, user, scope, at, client)
+            # Public clients don't get refresh tokens
+            if client.client_type != 1:
+                rt = self.create_refresh_token(request, user, scope, at, client)
 
         return self.access_token_response(at)
 
@@ -522,7 +560,7 @@ class AccessToken(OAuthView, Mixin):
 
     def get(self, request):
         """
-        As per :draft:`3.2` the token endpoint *only* supports POST requests.
+        As per :rfc:`3.2` the token endpoint *only* supports POST requests.
         Returns an error response.
         """
         return self.error_response({
@@ -531,7 +569,7 @@ class AccessToken(OAuthView, Mixin):
 
     def post(self, request):
         """
-        As per :draft:`3.2` the token endpoint *only* supports POST requests.
+        As per :rfc:`3.2` the token endpoint *only* supports POST requests.
         """
         if constants.ENFORCE_SECURE and not request.is_secure():
             return self.error_response({
